<?php
/**
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 *  @author Sadad Developer <developer@sadad.qa>
 *  @copyright  Copyright 2023 Sadad developer
 *  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/

class SadadqaValidationModuleFrontController extends ModuleFrontController
{
    public function postProcess()
    {
		$retResultAry = array();
        $key_id = Configuration::get('SADADQA_KEY_ID');
        $key_secret = trim(Configuration::get('SADADQA_KEY_SECRET'));
        $cbUrl = trim(Configuration::get('SADADQA_CALLBACK_URL'));
        $mlng = strtolower(Configuration::get('SADAD_WB_LANG'));
		$chkType = Configuration::get('SADAD_CHECKOUTTYPE');
		
	
		$chkType2 = Configuration::get('SADAD_CHECKOUTTYPE2');
		$chkLoader =  Configuration::get('SADAD_CHECKOUT_LOADER');
        $cart = $this->context->cart;
		$callBackUrl = Tools::getHttpHost(true) . __PS_BASE_URI__;
        if (($cart->id_customer === 0) or ($cart->id_address_delivery === 0)
        or ($cart->id_address_invoice === 0) or (!$this->module->active)) {
			$retResultAry['status'] = 'failed';
			$retResultAry['msg'] = 'This payment method is not active. Please try with different payment method';
            
			echo json_encode($retResultAry);
            exit;
        }
        $authorized = false;
        foreach (Module::getPaymentModules() as $module) {
            if ($module['name'] == 'sadadqa') {
                $authorized = true;
                break;
            }
        }
        if (!$authorized) {
            die('This payment method is not available.');
        }
        $customer = new Customer($cart->id_customer);
        if (!Validate::isLoadedObject($customer)) {
            
            $retResultAry['status'] = 'failed';
			$retResultAry['msg'] = 'Customer session time out.';
            
			echo json_encode($retResultAry);
            exit;
        }
        if ($this->context->currency->iso_code != 'QAR') {
            
             $retResultAry['status'] = 'failed';
			$retResultAry['msg'] = 'Qatari Rial(QAR) is the only supported currency for this payment method.';
            
			echo json_encode($retResultAry);
            exit;
        }
		
		
        try {
            $oid = $cart->id;
            $theAddress = new Address((int)$cart->id_address_invoice);
            $amount = $cart->getOrderTotal(true, Cart::BOTH);
            $redirect_url = $callBackUrl. 'index.php?fc=module&module=sadadqa&controller=webhook';
            $email = $customer->email;
            $mobile_no = (empty($theAddress->phone)) ? $theAddress->phone_mobile : $theAddress->phone;
			$txnDate = date('Y-m-d H:i:s');
			$sadad_checksum_array = array();
			$sadad__checksum_data = array();
            $i = 0;
            foreach ($cart->getProducts() as $item) {
                $qty = (string)$item['quantity'];
                $iTot = number_format((float)$item['total'], 2, '.', '');
                $iPric = number_format((float)$item['price_wt'], 2, '.', '');
                $inm = html_entity_decode($item['name']);
				
				$sadad_checksum_array['productdetail'][$i]['order_id']    = $oid;
				$sadad_checksum_array['productdetail'][$i]['amount']      = $iPric;
				$sadad_checksum_array['productdetail'][$i]['quantity']    = $qty;
                $i++;
            }
			
			$sadad_checksum_array['merchant_id']                     = $key_id;
            $sadad_checksum_array['ORDER_ID']                        = $oid;
            $sadad_checksum_array['WEBSITE']                         = $cbUrl;
            $sadad_checksum_array['TXN_AMOUNT']                      = number_format((float)$amount, 2, '.', '');
			$sadad_checksum_array['VERSION']                         = '1.1';
            $sadad_checksum_array['CUST_ID']                         = $email;
            $sadad_checksum_array['EMAIL']                           = $email;
            $sadad_checksum_array['MOBILE_NO']                       = $mobile_no;
            $sadad_checksum_array['SADAD_WEBCHECKOUT_PAGE_LANGUAGE'] = $mlng;
            $sadad_checksum_array['CALLBACK_URL']                    = $redirect_url;
            $sadad_checksum_array['txnDate']                         = $txnDate;
			
            
            //if webcheckout 2.2, add these fields to checksum
            if ($chkType == 2) {
                $sadad_checksum_array['SADAD_WEBCHECKOUT_HIDE_LOADER'] = $chkLoader;
                $sadad_checksum_array['showdialog']                    = (string) $chkType2;
            }
			$sAry1 = array();
			$sadad_checksum_array1 = array();
			foreach($sadad_checksum_array as $pK => $pV){
				if($pK=='checksumhash') continue;
				if(is_array($pV)){
					$prodSize = sizeof($pV);
					for($i=0;$i<$prodSize;$i++){
						foreach($pV[$i] as $innK => $innV){
							$sAry1[] = "<input type='hidden' name='productdetail[$i][". $innK ."]' value='" . trim($innV) . "'/>";
							$sadad_checksum_array1['productdetail'][$i][$innK]    = trim($innV);
						}
					}
				} else {
					$sAry1[] = "<input type='hidden' name='". $pK ."' id='". $pK ."' value='" . trim($pV) . "'/>";
					$sadad_checksum_array1[$pK]    = trim($pV);
				}
			}
            
            $sadad__checksum_data['postData']  = $sadad_checksum_array1;
            $sadad__checksum_data['secretKey'] = $key_secret;
            $checksum                          = $this->getChecksumFromString(json_encode($sadad__checksum_data), $key_secret . $key_id);
            $sAry1[]                = "<input type='hidden' name='checksumhash' value='" . $checksum . "'/>";
			
			ob_start();
           if ($chkType == 1) {
                $action_url = 'https://sadadqa.com/webpurchase';
                
                echo '<form action="' . $action_url . '" method="post" id="sadad_payment_form" name="gosadad">
                    ' . implode('', $sAry1) . '
                    <script type="text/javascript">
                        document.gosadad.submit();
                    </script>
                    
                </form>';
            } else {
				$action_url = 'https://secure.sadadqa.com/webpurchasepage';
                
                echo '<form action="' . $action_url . '" method="post" id="paymentform" name="paymentform" data-link="' . $action_url . '">
                    ' . implode('', $sAry1) . '
                </form>';
                
?>
               <!--<script src="https://ajax.googleapis.com/ajax/libs/jquery/3.5.1/jquery.min.js"></script>
                <script src="https://cdnjs.cloudflare.com/ajax/libs/popper.js/1.12.9/umd/popper.min.js" integrity="sha384-ApNbgh9B+Y1QKtv3Rn7W3mgPxhU9K/ScQsAP7hUibX39j7fakFPskvXusvfa0b4Q" crossorigin="anonymous"></script>
                
                <script src="https://maxcdn.bootstrapcdn.com/bootstrap/4.0.0/js/bootstrap.min.js" integrity="sha384-JZR6Spejh4U02d8jOt6vLEHfe/JQGiRRSQQxSfFWpi1MquVdAyjUar5+76PVCmYl" crossorigin="anonymous"></script>
                
                <script src="https://cdnjs.cloudflare.com/ajax/libs/bootstrap-modal/2.2.6/js/bootstrap-modal.min.js"></script>
                
                <script src="https://cdnjs.cloudflare.com/ajax/libs/bootstrap-modal/2.2.6/js/bootstrap-modalmanager.js"></script>
                
                <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/bootstrap-modal/2.2.6/css/bootstrap-modal.min.css" crossorigin="anonymous">
                
                <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/bootstrap-modal/2.2.6/css/bootstrap-modal-bs3patch.min.css" crossorigin="anonymous">-->
                
                    <style>
                        .close-btn{
                            height: auto;
                            width: auto;
                            -webkit-appearance: none !important;
                            background: none !important;
                            border: 0px;
                            position: absolute;
                            right: 10px;
                            z-index: 11;
                            cursor: pointer;
                            outline: 0px !important;
                            box-shadow: none;
                            top: 15px;
                        }
                        .close, .close:hover{
                            color: #000;
                            font-size:30px;
                        }
                        .modal-body{
                            padding: 0px;
                            border-radius: 15px;
                        }
                        #onlyiframe{
                         width:100% !important;
                         height:100vh !important;
                         overflow: hidden !important;
                         border:0;
                        
                         top: 0;
                         left: 0;
                         bottom: 0;
                         right: 0;
                              
                        }
                        #includeiframe{
                            height:100vh !important;
                         overflow: hidden !important;
                        border:0;
						width:100%;
                        }
                        .modal-backdrop {
                           background-color: #000 !important;
                        }
						#exampleModal .modal-content{
							background: #fff !important;
						}
                    </style>
                    <!-- Modal -->
                    <div id="container_div_sadad">
                    <div class="modal fade not_hide_sadad" id="exampleModal" tabindex="-1" role="dialog" aria-labelledby="exampleModalLabel" aria-hidden="true" data-backdrop='static'>
                        <div class="modal-dialog" role="document">
                            <div class="modal-content">
                                <button type="button" class="close-btn" data-dismiss="modal" aria-label="Close">
                                    <span class="close">&times;</span>
                                </button>
                                <div class="modal-body">
                                    <iframe name="includeiframe" id="includeiframe" frameborder="0" scrolling="no"></iframe>
                                </div>
                            </div>
                        </div>
                    </div>
                    <iframe name="onlyiframe" id="onlyiframe" border="0" class="not_hide_sadad" frameborder="0" scrolling="no"></iframe>
                    </div>
                    
                    <script>
                        function closemodal()
                        {
							window.location.href = '<?php echo $callBackUrl; ?>index.php?controller=order&step=1';
                            $('#exampleModal').modal('hide');
                        }
                        jQuery(document).ready(function($){
                        
                                if ($('#showdialog').val() == 1) {
                                
                                      $('#exampleModal').modal('show');
                                      $('#paymentform').attr('target', 'includeiframe').submit();
                                      $('#onlyiframe').hide();
                                    
                                } else {
                                     $('#exampleModal').modal('hide');
                                     $('#paymentform').attr('target', 'onlyiframe').submit();
                                    
                                }
                             $('iframe').load(function() {
                                    $(this).height( $(this).contents().find("body").height() );
                                    if(this.contentWindow.location=='<?php
                echo $redirect_url;
?>'){
									
									$(this).hide();
									window.location.href = '';
                                        
                                    }
                                });    
                        });
                    </script>

                <?php
			}
			$msgDataa = ob_get_contents();
			ob_end_clean();
			
			 $retResultAry['status'] = 'success';
			$retResultAry['msg'] = $msgDataa;
            if(isset($_GET['isAjax']) && $_GET['isAjax']==1){
				echo json_encode($retResultAry);
			} else {
				echo $msgDataa;
			}
            exit;
        } catch (Exception $e) {
            $error = $e->getMessage();
            Logger::addLog("Payment Failed for Order# " . $cart->id . "Error: " . $error, 4);
           
             $retResultAry['status'] = 'failed';
			$retResultAry['msg'] = 'Something went wrong. Please try after sometime.';
            
			echo json_encode($retResultAry);
            exit;
            
        }
    }
	
	

		private function getChecksumFromString($str, $key) {
	
		
	
		$salt = $this->generateSalt_e(4);
	
		$finalString = $str . "|" . $salt;
	
		$hash = hash("sha256", $finalString);
	
		$hashString = $hash . $salt;
	
		$checksum = $this->encrypt_e($hashString, $key);
	
		return $checksum;
	
	}
	
	
	
	private function generateSalt_e($length) {
	
		$random = "";
	
		srand((double) microtime() * 1000000);
	
	
	
		$data = "AbcDE123IJKLMN67QRSTUVWXYZ";
	
		$data .= "aBCdefghijklmn123opq45rs67tuv89wxyz";
	
		$data .= "0FGH45OP89";
	
		for ($i = 0; $i < $length; $i++) {
	
			$random .= substr($data, (rand() % (strlen($data))), 1);
	
		}
	
		return $random;
	
	}



	private function encrypt_e($input, $ky) {
	
	$ky = html_entity_decode($ky);
	
	$iv = "@@@@&&&&####$$$$";
	
	$data = openssl_encrypt($input, "AES-128-CBC", $ky, 0, $iv);
	
	return $data;
	
	}
	
	/*
	

*/
}
