<?php
/**
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 *  @author Sadad Developer <developer@sadad.qa>
 *  @copyright  Copyright 2023 Sadad developer
 *  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/

require_once _PS_MODULE_DIR_ . 'sadadqa/sadadqa-webhook.php';

class SadadqaWebhookModuleFrontController extends ModuleFrontController
{
    public function postProcess()
    {
        try {
            $sdqWebhook = new SdqWebhook();
            $test = $sdqWebhook->process();
			
            if (is_array($test)) {
                $ret = $this->module->validateOrder(
                    $test['crt']->id,
                    (int)Configuration::get('PS_OS_PAYMENT'),
                    $test['crt']->getOrderTotal(true, Cart::BOTH),
                    'Sadad Payment',
                    'Payment by Sadad',
                    $test['extra'],
                    null,
                    false,
                    $test['secureKey']
                );
                Logger::addLog(
                    "Success Order#" . $test['crt']->id . $ret . ".Sadad: " . $test['extra']['transaction_id'],
                    1
                );
                $query = http_build_query(
                    [
                        'controller' => 'order-confirmation',
                        'id_cart' => (int)$test['crt']->id,
                        'id_module' => (int)$this->module->id,
                        'id_order' => $this->module->currentOrder,
                        'key' => $test['secureKey'],
                    ],
                    '',
                    '&'
                );
                $url = 'index.php?' . $query;
				
                Tools::redirect($url);
                exit;
            } else {
                $this->errors[] = $this->l($test);
                $this->redirectWithNotifications('index.php?controller=order&step=1');
                exit;
            }
        } catch (Exception $e) {
            $error = $e->getMessage();
            Logger::addLog("Error: " . $error, 4);
        }
        Tools::redirect('index.php');
        exit;
    }
}
