<?php
/**
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 *  @author Sadad Developer <developer@sadad.qa>
 *  @copyright  Copyright 2023 Sadad developer
 *  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;
use PrestaShop\PrestaShop\Adapter\Cart\CartPresenter;

class Sadadqa extends PaymentModule
{
    private $html = '';
    private $KEY_ID = null;
    private $KEY_SECRET = null;
    private $SADADQA_CALLBACK_URL = null;
    private $SADAD_WB_LANG = null;
	private $SADAD_CHECKOUTTYPE = '1';
	private $SADAD_CHECKOUTTYPE2 = null;
	private $SADAD_CHECKOUT_LOADER = null;
    private $postErrors = [];
    public function __construct()
    {
        $this->controllers = ['validation'];
        $this->name = 'sadadqa';
        $this->displayName = 'Sadad QAR';
        $this->tab = 'payments_gateways';
        $this->version = '2.0.0';
        $this->need_instance = 1;
        $this->ps_versions_compliancy = ['min' => '1.7.0.0', 'max' => _PS_VERSION_];
        $this->display = true;
        $this->author = 'Sadad Developer';
        $this->module_key = '58aba174b7f3393b9673f865c50fad43';
        $config = Configuration::getMultiple(['SADADQA_KEY_ID', 'SADADQA_KEY_SECRET',
        'SADADQA_CALLBACK_URL', 'SADAD_WB_LANG','SADAD_CHECKOUTTYPE','SADAD_CHECKOUTTYPE2','SADAD_CHECKOUT_LOADER']);
        if (array_key_exists('SADADQA_KEY_ID', $config)) {
            $this->KEY_ID = $config['SADADQA_KEY_ID'];
        }
        if (array_key_exists('SADADQA_KEY_SECRET', $config)) {
            $this->KEY_SECRET = $config['SADADQA_KEY_SECRET'];
        }
        if (array_key_exists('SADADQA_CALLBACK_URL', $config)) {
            $this->SADADQA_CALLBACK_URL = $config['SADADQA_CALLBACK_URL'];
        }
        if (array_key_exists('SADAD_WB_LANG', $config)) {
            $this->SADAD_WB_LANG = $config['SADAD_WB_LANG'];
        }
		if (array_key_exists('SADAD_CHECKOUTTYPE', $config)) {
            $this->SADAD_CHECKOUTTYPE = $config['SADAD_CHECKOUTTYPE'];
        }
        if (array_key_exists('SADAD_CHECKOUTTYPE2', $config)) {
            $this->SADAD_CHECKOUTTYPE2 = $config['SADAD_CHECKOUTTYPE2'];
        }
        if (array_key_exists('SADAD_CHECKOUT_LOADER', $config)) {
            $this->SADAD_CHECKOUT_LOADER = $config['SADAD_CHECKOUT_LOADER'];
        }
        parent::__construct();
        $this->page = basename(__FILE__, '.php');
        $this->description = $this->l('Accept payments with Credit card');
        if ($this->KEY_ID === null or $this->KEY_SECRET === null) {
            $this->warning = $this->l('Sadad ID and Secret key must be configured to use this module');
        }
    }
    public function getContent()
    {
        $this->html = '<h2>' . $this->displayName . '</h2>';
        if (Tools::isSubmit('btnSubmit')) {
            $this->postValidation();
            if (empty($this->postErrors)) {
                $this->postProcess();
            } else {
                foreach ($this->postErrors as $err) {
                    $this->html .= "<div class='alert error'>ERROR: {$err}</div>";
                }
            }
        } else {
            $this->html .= "<br />";
        }
        $this->displaysadadqa();
        $this->displayForm();
        return $this->html;
    }
    private function displayForm()
    {
        $modsadadqa = $this->l('Sadad QAR Setup');
        $modsadadqaDesc = $this->l('Please specify the Sadad Merchant ID and Secret key.');
        $modClientLabelKeyId = $this->l('Sadad Merchant ID');
        $modClientLabelKeySecret = $this->l('Sadad Secret key');
        $keyId = $this->KEY_ID;
        $modClientValueKeySecret = $this->KEY_SECRET;
        $cbul = $this->SADADQA_CALLBACK_URL;
        $mLang = $this->SADAD_WB_LANG;
		$chkType = $this->SADAD_CHECKOUTTYPE;
		$chkType2 = $this->SADAD_CHECKOUTTYPE2;
		$chkLoader = $this->SADAD_CHECKOUT_LOADER;
		$type2Hide = ($chkType == '1' or empty($chkType))?"style='display:none;'":"";
        $this->html .= "
        <br />
        <br />
        <p><form action='{$_SERVER['REQUEST_URI']}' method='post'>
                <fieldset>
                <legend>{$modsadadqa}</legend>
                        <table border='0' width='500' cellpadding='' cellspacing='0' class='tbl_sadad' id='form'>
                                <tr>
                                        <td colspan='2'>
                                                {$modsadadqaDesc}<br /><br />
                                        </td>
                                </tr>
                                <tr>
                                        <td width='130'>{$modClientLabelKeyId}</td>
                                        <td>
                                         <input type='text' name='KEY_ID' value='{$keyId}' style='width: 300px;'/>
                                        </td>
                                </tr>
                                <tr>
                                        <td width='130'>{$modClientLabelKeySecret}</td>
                                        <td>
											<input type='text' name='KEY_SECRET' value='{$modClientValueKeySecret}' style='width: 300px;' />     
                                        </td>
                                </tr>
								<tr>
                                        <td width='130'>Website</td>
                                        <td>
                                         <input type name='SADADQA_CALLBACK_URL' value='{$cbul}' style='width: 300px;'>
									      
                                        </td>
                                </tr>
                                <tr>
                                        <td width='130'>Webcheckout Language</td>
                                        <td>
                                         <select name='SADAD_WB_LANG' style='width: 300px;'>
									      <option value='ARB' " . (($mLang == 'ARB') ? 'selected' : '') . ">Arabic</option>
										  <option value='ENG' " . (($mLang == 'ENG') ? 'selected' : '') . ">English</option>
										 </select>
                                        </td>
                                </tr>
								<tr>
                                        <td width='130'>Checkout type</td>
                                        <td>
                                         <select name='SADAD_CHECKOUTTYPE' style='width: 300px;'>
									      <option value='1' " . (($chkType == '1') ? 'selected' : '') . ">Web checkout</option>
										  <option value='2' " . (($chkType == '2') ? 'selected' : '') . ">Web checkout 2.2</option>
										 </select>
                                        </td>
                                </tr>
								<tr id='tr_chkouttype2' ". $type2Hide .">
                                        <td width='130'>Checkout 2.2 Type</td>
                                        <td>
                                         <select name='SADAD_CHECKOUTTYPE2' style='width: 300px;'>
									      <option value='1' " . (($chkType2 == '1') ? 'selected' : '') . ">Modal popup</option>
										  <option value='2' " . (($chkType2 == '2') ? 'selected' : '') . ">Iframe</option>
										 </select>
                                        </td>
                                </tr>
								<tr>
                                        <td width='130'>Hide Loader</td>
                                        <td>
                                         <select name='SADAD_CHECKOUT_LOADER' style='width: 300px;'>
									      <option value='YES' " . (($chkLoader == 'YES') ? 'selected' : '') . ">YES</option>
										  <option value='NO' " . (($chkLoader == 'NO') ? 'selected' : '') . ">NO</option>
										 </select>
                                        </td>
                                </tr>
                                <tr>
                                        <td colspan='2' align='center'>
                                         <input class='button' name='btnSubmit' value='Update' type='submit' />
                                        </td>
                                </tr>
                        </table>
                </fieldset>
        </form>
		<script>
			$('select[name=SADAD_CHECKOUTTYPE]').on('change',function(){
				$('#tr_chkouttype2').toggle();
			});
		</script>
		<style>
			.tbl_sadad tr,td{
				padding: 8px;
			}
		</style>
        </p>
        <br />";
    }
    public function install()
    {
        if (parent::install() && $this->registerHook('header')
            && $this->registerHook('orderConfirmation') && $this->registerHook('paymentOptions')
            && $this->registerHook('paymentReturn')) {
            return true;
        }
        return false;
    }
    public function hookHeader()
    {
        if (Tools::getValue('controller') == "order") {
            $this->context->controller->registerJavascript(
                'sadadqa-checkout-local-script',
                'modules/' . $this->name . '/views/js/script.js',
                ['position' => 'bottom', 'priority' => 30]
            );
            Media::addJsDef(['sadadqa_checkout_vars' => ['siteUrl' => Tools::getHttpHost(true) . __PS_BASE_URI__]]);
        }
    }
    public function hookOrderConfirmation($params)
    {
        $order = $params['order'];
        if ($order) {
            if ($order->module === $this->name) {
                $payments = $order->getOrderPayments();
                if (count($payments) >= 1) {
                    $payment = $payments[0];
                    $paymentId = $payment->transaction_id;
                    return "Sadad transaction Id: <code>$paymentId</code>";
                }
                return;
            }
        }
    }
    public function uninstall()
    {
        Configuration::deleteByName('SADADQA_KEY_ID');
        Configuration::deleteByName('SADADQA_KEY_SECRET');
        Configuration::deleteByName('SADADQA_CALLBACK_URL');
        Configuration::deleteByName('SADAD_WB_LANG');
		Configuration::deleteByName('SADAD_CHECKOUTTYPE');
		Configuration::deleteByName('SADAD_CHECKOUTTYPE2');
		Configuration::deleteByName('SADAD_CHECKOUT_LOADER');
        return parent::uninstall();
    }
    public function hookPaymentOptions($params)
    {
        $option = new PaymentOption();
        $option->setModuleName('sadadqa')->setLogo('./modules/sadadqa/views/img/logo.png')
        ->setAction($this->context->link->getModuleLink('sadadqa', 'validation', [], true))
        ->setCallToActionText('Pay with credit/debit card')->setAdditionalInformation('');
        return [$option];
    }
    public function hookPaymentReturn($params)
    {
        if (!$this->active) {
            return;
        }
        if ((!isset($params['order'])) or ($params['order']->module != $this->name)) {
            return false;
        }
        if ((isset($params['order'])) && (Validate::isLoadedObject($params['order']))
            && (isset($params['order']->valid))) {
            $this->smarty->assign(['id_order' => $params['order']->id, 'valid' => $params['order']->valid,]);
        }
        if ((isset($params['order']->reference)) and (!empty($params['order']->reference))) {
            $this->smarty->assign('reference', $params['order']->reference);
        }
        $this->smarty->assign(
            [
                'shop_name' => $this->context->shop->name,
                'reference' => $params['order']->reference,
                'contact_url' => $this->context->link->getPageLink('contact', true),
            ]
        );
        return $this->fetch('module:sadadqa/views/templates/hook/payment_return.tpl');
    }
    private function postValidation()
    {
        if (Tools::isSubmit('btnSubmit')) {
            $keyId = Tools::getValue('KEY_ID');
            $keySecret = Tools::getValue('KEY_SECRET');
            if (empty($keyId)) {
                $this->postErrors[] = $this->l('Your Key Id is required.');
            }
            if (empty($keySecret)) {
                $this->postErrors[] = $this->l('Your Key Secret is required.');
            }
        }
    }
    private function postProcess()
    {
        if (Tools::isSubmit('btnSubmit')) {
            Configuration::updateValue('SADADQA_KEY_ID', Tools::getValue('KEY_ID'));
            Configuration::updateValue('SADADQA_KEY_SECRET', Tools::getValue('KEY_SECRET'));
            Configuration::updateValue('SADADQA_CALLBACK_URL', Tools::getValue('SADADQA_CALLBACK_URL'));
            Configuration::updateValue('SADAD_WB_LANG', Tools::getValue('SADAD_WB_LANG'));
			Configuration::updateValue('SADAD_CHECKOUTTYPE', Tools::getValue('SADAD_CHECKOUTTYPE'));
			Configuration::updateValue('SADAD_CHECKOUTTYPE2', Tools::getValue('SADAD_CHECKOUTTYPE2'));
			Configuration::updateValue('SADAD_CHECKOUT_LOADER', Tools::getValue('SADAD_CHECKOUT_LOADER'));
            $this->KEY_ID = Tools::getValue('KEY_ID');
            $this->KEY_SECRET = Tools::getValue('KEY_SECRET');
            $this->SADADQA_CALLBACK_URL = Tools::getValue('SADADQA_CALLBACK_URL');
            $this->SADAD_WB_LANG = Tools::getValue('SADAD_WB_LANG');
			$this->SADAD_CHECKOUTTYPE = Tools::getValue('SADAD_CHECKOUTTYPE');
			$this->SADAD_CHECKOUTTYPE2 = Tools::getValue('SADAD_CHECKOUTTYPE2');
			$this->SADAD_CHECKOUT_LOADER = Tools::getValue('SADAD_CHECKOUT_LOADER');
        }
        $ok = $this->l('Ok');
        $updated = $this->l('Settings Updated');
        $this->html .= "<div class='conf confirm'><img src='../img/admin/enabled.gif' alt='{$ok}' />{$updated}</div>";
    }
    private function displaysadadqa()
    {
        $modDesc = $this->l('This module allows you to accept payments using Sadad.');
        $this->html .= "<img src='../modules/sadadqa/views/img/logo.png' style='float:left; margin-right:15px;' /><br />
            <b>{$modDesc}</b>
            <br />";
    }
}
